<?php

namespace Drupal\mcapi\Form;

use Drupal\user\Entity\User;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;

/**
 * Builder for currency entity form.
 */
class CurrencyForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    $currency = $this->entity;

    $form['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Name of currency'),
      '#description' => $this->t('Use the plural'),
      '#default_value' => $currency->label(),
      '#size' => 40,
      '#maxlength' => 80,
      '#required' => TRUE,
      '#weight' => 0,
    ];
    $form['description'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Description of the currency'),
      '#description' => $this->t('Why is this currency credible? Who guarantees it? What is its value? What is its story?'),
      '#default_value' => $currency->description,
      '#weight' => 1,
    ];

    if ($currency->id()) {
      $form['id'] = [
        '#type' => 'value',
        '#value' => $currency->id(),
      ];
    }
    else {
      $form['id'] = [
        '#type' => 'machine_name',
      // This is the max length of the worth->curr_id column.
        '#maxlength' => 8,
        '#machine_name' => [
          'exists' => '\Drupal\mcapi\Entity\Currency::load',
          'source' => ['name'],
        ],
        '#description' => $this->t('A unique machine-readable name for this Currency. It must only contain lowercase letters, numbers, and underscores.'),
      ];
    }
    // Default option for before any roles have been defined.
    $form['uid'] = [
      '#type' => 'value',
      '#value' => 1,
    ];

    if ($roles = user_roles(TRUE, 'manage mcapi')) {
      $admins = \Drupal::entityTypeManager()
        ->getStorage('user')
        ->loadByProperties(['roles' => array_keys($roles)]);
      if (count($admins)) {
        $options = [1 => User::load(1)->getDisplayName()];
        foreach ($admins as $uid => $user) {
          $options[$uid] = $user->getDisplayName();
        }
        $currentUser = \Drupal::currentUser();
        $form['uid'] = [
          '#title' => $this->t('Comptroller'),
          '#description' => $this->t("Any user with 'Manage community accounting' role"),
          '#type' => 'select',
          '#options' => $options,
          '#default_value' => $currency->uid,
          //'#access' => $currentUser->hasPermission('configure mcapi') || $currentUser->id() == $currency->getOwner()->id()
        ];
      }
    }
    $form['#attached']['library'][] = 'mcapi/currency-form';


    $form['help'] = [
      '#title' => $this->t('Help on currency types'),
      '#description' => $this->t('How to configure your currency'),
      '#type' => 'radios',
      '#options' => [
        '' => t('No help needed'),
        'lets' => t('LETS'),
        'mutualcredit' => t('Mutual Credit'),
        'timebank' => t('Time bank'),
        'vouchers' => t('Vouchers backed by state money'),

      ],
      '#default_value' => '',
      '#weight' => 4,
    ];
    $lets = [
      $this->t('Some LETS are strict about credit and limits - see mutual credit. Others are less strict. See time banks.'),
    ];
    $mutualcredit = [
      $this->t('All accounts must stay within a tolerable range of zero.'),
      $this->t('There is no acccount for issuing, but you may have an admin account with a wider tolerance.'),
      $this->t('Use the Balanced limits plugin'),
    ];
    $timebank = [
      $this->t('Timebanks usual both issue hours to volunteers and allow members to trade amongst themselves.'),
      $this->t('Average members therefore accrue in the long term, more hours than they can spend'),
      $this->t('Some members may also be allowed negative balances.'),
      $this->t('If you need balance limits at all, use the explicit plugin and then make exceptions per member.'),
    ];
    $vouchers = [
      $this->t('When a local currency is backed by the national currency, that means for every unit issued, you have one unit in in reserve, in cash or in the bank.'),
      $this->t("On this system you need one account with a max balance of zero, and all other accounts have a min balance of zero."),
      $this->t('Then when someone buys a voucher you pay into their account from the first account.'),
    ];
    foreach (array_filter(array_keys($form['help']['#options'])) as $key) {
      $form[$key] = [
        '#type' => 'container',
        '#children' => implode(" ", $$key),
        '#weight' => 3,
        '#states' => [
          'visible' => [
            ':input[name="help"]' => ['value' => $key],
          ],
        ],
        '#style' => 'float:right;'
      ];
    }


    $serials = $currency->isNew() ?
      0 :
      $currency->transactionCount(['worth.value' => 0]);

    $form['zero'] = [
      '#title' => $this->t('Allow zero transactions'),
      '#description' => $this->t('Enter an HMTL snippet to show when a transaction is worth zero units. Leave blank to disallow zero transactions.'),
      '#type' => 'checkbox',
      '#default_value' => $currency->zero,
      // This is required if any existing transactions have zero value.
      '#weight' => 7,
    ];
    if ($serials) {
      $form['zero']['#required'] = TRUE;
      $form['zero']['#description'] .= $this->t("Zero transaction already exist so this field is required");
    }
    $help[] = $this->t('E.g. for Hours, Minutes put Hr99:59/4mins, for dollars and cents put $9.99; for loaves put 99 loaves.');
    $help[] = $this->t('The first number is always a string of zeros showing the number of characters (powers of ten) the widget will allow.');
    $help[] = $this->t('The optional /n at the end will render the final number widget as a dropdown field showing intervals, in the example, of 15 mins.');
    $help[] = $this->t('Special effects can be accomplised with HTML & css.');
    $form['format'] = [
      '#title' => $this->t('Format'),
      '#description' => implode(' ', $help),
      '#type' => 'textfield',
      '#default_value' => implode('', $currency->format),
      '#element_validate' => [
        [$this, 'validateFormatElement'],
      ],
      '#max_length' => 32,
      '#size' => 32,
      '#weight' => 10
    ];
    $form['#attached']['library'][] = 'mcapi/currency-form';
    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $currency = $this->entity;
    $status = $currency->save();

    if ($status == SAVED_UPDATED) {
      $this->messenger()->addStatus(t('Currency %label has been updated.', ['%label' => $currency->label()]));
    }
    else {
      $this->messenger()->addStatus(t('Currency %label has been added.', ['%label' => $currency->label()]));
    }
    $form_state->setRedirect('entity.mcapi_currency.collection');
  }

  /**
   * Element validation callback.
   */
  public function validateFormatElement(&$element, FormStateInterface $form_state) {
    $array = self::transformFormat($element['#value']);
    if (is_array($array)) {
      $form_state->setValue('format', $array);
    }
    else {
      $form_state->setErrorByName('format', t('Bad Format'));
    }
  }

  /**
   * Explode the format string into alternating numbers and template chunks.
   *
   * @param string $string
   *   The input from the currency form 'format' field.
   *
   * @return array
   *   With values alternating string / number / string / number etc.
   */
  public static function transformFormat($string) {
    // A better regular expression would make this function much shorter
    // ($not_nines) | ([nines] | [not nines])+ | (/nine)? | (not_nines) ?
    $patterns = [
      '/^([^9]*)(9+)([^0-9]*)$/', //integer e.g. '9 loaves' or L9
      '/^([^9]*)(9+)([:,. ])([0-9\/]*)(.*)$/', //units and subunits e.g. $99.99 or Hr99:59/4mins
    ];
    foreach ($patterns as $pattern) {
      $parts = [];
      if (preg_match_all($pattern, $string, $parts)){
        break;
      }
    }
    if ($parts) {
      array_shift($parts);
      $format = [];
      foreach($parts as $array) {
        //$format[] = $array[0] ? : 9;
        if ($array[0]) {
          $format[] = $array[0];
        }
      }
      // Ensure the first value of the result array corresponds to a template
      // string, not a numeric string if the format string started with a number.
      if (is_numeric(substr($string, 0, 1))) {
        array_unshift($format, '');
      }
      // If the last value of $combo is empty remove it.
      if (end($format) == '') {
        array_pop($format);
      }
      return $format;
    }
  }

}
